################################################
#Author      : Annette Janssen & Bram Droppers
#Institution : Wageningen University
#Date        : January 2021
################################################

#CALCULATE TEMPERATURE VARIABLES###############################################################################
calculateTemperatureVariables = function(nc)
{
  temperature       = ncvar_get(nc, nc$var$OUT_LAKE_LAYER_TEMP, start = c(1,1,i), count = c(-1,-1,1))
  temperature.ice   = ncvar_get(nc, nc$var$OUT_LAKE_ICE_TEMP, start = c(1,i), count = c(-1,1))
  temperature.snow  = ncvar_get(nc, nc$var$OUT_SNOW_SURF_TEMP, start = c(1,i), count = c(-1,1))
  depth             = ncvar_get(nc, nc$var$OUT_LAKE_DEPTH, start = c(1,i), count = c(-1,1))

  nnode = floor((depth - 0.6) * 2) + 1
  nnode[depth < 0.6] = 0
  
  density = 999.842594 + 
    (6.793952 * 10^-2 * temperature) - 
    (9.095290 * 10^-3 * temperature ^ 2) + 
    (1.001685 * 10^-4 * temperature ^ 3) - 
    (1.120083 * 10^-6 * temperature ^ 4) +
    (6.536336 * 10^-9 * temperature ^ 5) # Based on ISIMIP protocol
  
  # Analyse stratification
  stratified = rep(0, nc$dim$lake_class$len)
  thermocline.depth = rep(0, nc$dim$lake_class$len)
  temperature.surf = rep(0, nc$dim$lake_class$len)
  temperature.bot = rep(0, nc$dim$lake_class$len)
  temperature.epi = rep(0, nc$dim$lake_class$len)
  temperature.hyp = rep(0, nc$dim$lake_class$len)
  
  for(j in 1:nc$dim$lake_class$len){
    temperature[j, (nnode[j] + 1):dim(temperature)[2]] = NA
    
    if(nnode[j] == 0){
      next
    }
    
    temperature.surf[j] = temperature[j,1]
    temperature.bot[j] = temperature[j, nnode[j]]
    
    if(nnode[j] == 1){
      next
    }
    
    density.surf = density[j, 1]
    density.bot = density[j, nnode[j]]
    if (abs(density.surf - density.bot) > 0.1) {
      # Based on ISIMIP protocol
      stratified[j] = 1
    }
    
    density.diff = diff(density[j,1:nnode[j]])
    density.diff = density.diff / node.depth[1:length(density.diff)]
    
    thermocline.node = which.max(abs(density.diff))
    thermocline.depth[j] = sum(node.depth[1:thermocline.node - 1]) + (node.depth[thermocline.node] / 2)
    
    temperature.epi[j] = mean(temperature[j,1:thermocline.node])
    temperature.hyp[j] = mean(temperature[j,thermocline.node:nnode[j]])
    
    
  }
  
  return(list(water.temp = temperature,
              ice.temp = temperature.ice,
              snow.temp = temperature.snow,
              water.temp.surf = temperature.surf, 
              water.temp.bot = temperature.bot,
              water.temp.epi = temperature.epi,
              water.temp.hyp = temperature.hyp,
              strat = stratified,
              thermo.depth = thermocline.depth,
              waterdepth=depth))
}


calculateEnergyVariables = function(nc)
{
  sensible = ncvar_get(nc, nc$var$OUT_SENSIBLE, start = c(1,i), count = c(-1,1))
  latent = ncvar_get(nc, nc$var$OUT_LATENT, start = c(1,i), count = c(-1,1))
  shortwave.down = ncvar_get(nc, nc$var$OUT_SWDOWN, start = c(1,i), count = c(-1,1))
  shortwave.net = ncvar_get(nc, nc$var$OUT_SWNET, start = c(1,i), count = c(-1,1))
  longwave.down = ncvar_get(nc, nc$var$OUT_LWDOWN, start = c(1,i), count = c(-1,1))
  longwave.net = ncvar_get(nc, nc$var$OUT_LWNET, start = c(1,i), count = c(-1,1))
  albedo = ncvar_get(nc, nc$var$OUT_ALBEDO, start = c(1,i), count = c(-1,1))
  evap =   ncvar_get(nc, nc$var$OUT_LAKE_EVAP, start = c(1,i), count = c(-1,1))
  prec = ncvar_get(nc, nc$var$OUT_PREC, start = c(1,i), count = c(-1,1))
  #ground.net = ncvar_get(nc, nc$var$OUT_GRND_FLUX, start = c(1,i), count = c(-1,1))
  
  heatflux.net = shortwave.net + longwave.net - sensible - latent
  
  return(list(sensible.flux = -sensible,
              latent.flux = -latent,
              sw.up = shortwave.down - shortwave.net,
              sw.down = shortwave.down, 
              lw.up = longwave.down - longwave.net,
              albedo = albedo,
              heat.flux = heatflux.net,
              evaporation = evap,
              precipitation = prec))
}


#CALCULATE STRATIFICATION class##############################################################################################################
calculateStratficationType = function(IceDays,IceThreshold, Strat,StratThreshold,Tepi,daysInYear,periodSmooth,meroThreshold)
{
  NumberOfLakes<-length(IceDays)
  OUTPUT.str<-Strat
  OUTPUT.mixingregime    <- array(0,c(NumberOfLakes))
  
  # ICE COVERAGE
  ICECOVERED             <- which(IceDays>=IceThreshold) 
  ICESOMETIMES           <- which(IceDays<IceThreshold & IceDays>(1-IceThreshold))
  ICEFREE                <- which(IceDays<=(1-IceThreshold)) 
  
  # MIXING
  extr <- periodSmooth-1
  OUTPUT.str2 <- as.data.frame(as.matrix(abind(OUTPUT.str,OUTPUT.str[,1:extr], along=2)))
  OUTPUT.str1<-as.data.frame(t(filter(x=t(OUTPUT.str2), filter=rep(x=1/(periodSmooth),times=(periodSmooth)), sides=2)))[,3:367]
  OUTPUT.str3<-OUTPUT.str1
  OUTPUT.str3[OUTPUT.str1>0]<-1
  
  strPeriod1<-t(cbind(0,OUTPUT.str3))
  strPeriod2=t(array(Reduce(x=strPeriod1, function(x, y) if (y == 0) 0 else x+y, accumulate=TRUE),dim(strPeriod1))) #cumsum but if 0 than restart counting
  strPeriod<-as.data.frame(strPeriod2[,2:366])  
  
  stratSrt <- OUTPUT.str3[,1:364]-OUTPUT.str3[,2:365]
  stratEnd <- OUTPUT.str3[,1:364]-OUTPUT.str3[,2:365]
  
  stratSrt[stratSrt<0]<-0
  stratEnd[stratEnd>0]<-0
  
  stratSrt<-rowSums(stratSrt)
  stratEnd<-rowSums(stratEnd)
  
  strPerioddiff<-  as.data.frame(sqrt(((strPeriod[,1:364]-strPeriod[,2:365])+1)*strPeriod[,1:364]))
  strPeriod1<-as.data.frame(t(apply(strPerioddiff, 1, sort)))[,363:364]
  
  PERMANENT_STRATIFIED          <- unique(c(ICECOVERED,which(rowSums(OUTPUT.str3) > (daysInYear-meroThreshold))))  # (Almost) PERMANENTLY STRATIFIED IN UNSMOOTHED FUNCTION
  MIXING_CONT                   <- setdiff(which(rowSums(OUTPUT.str3) == 0),ICECOVERED)                            # NEVER stratified
  MIXING_ONCE                   <- setdiff(which(strPeriod1[,2]>=StratThreshold & strPeriod1[,1]<=StratThreshold/2 
                                                 & (stratSrt == 1|stratEnd==1 )),PERMANENT_STRATIFIED)             # ONCE STRATIFIED 
  MIXING_TWICE                  <- setdiff(which(strPeriod1[,2]>=StratThreshold & strPeriod1[,1]>=StratThreshold/2 
                                                 & (stratSrt == 2|stratEnd==2 )),PERMANENT_STRATIFIED)             # TWICE STRATIFIED
  MIXING_MULT                   <- setdiff(setdiff(setdiff(setdiff(1:NumberOfLakes,PERMANENT_STRATIFIED),MIXING_ONCE),
                                                   MIXING_TWICE),MIXING_CONT)                                 # ALL OTHER LAKES
  
 
  # TEMPERATURE
  WARMS4C_N              <- which(rowMeans(Tepi)<=4)
  WARMS4C_Y              <- which(rowMeans(Tepi)>4)

  #TYPE OF MIXING REGIME
  # 1 = POLYMICTIC_COLD (CONTINUOUS)
  POLYMICTIC_CONT_COLD  <- intersect(MIXING_CONT,WARMS4C_N)
  OUTPUT.mixingregime[POLYMICTIC_CONT_COLD] <-1
  
  # 2 = POLYMICTIC_WARM (CONTINUOUS)
  POLYMICTIC_CONT_WARM  <- intersect(MIXING_CONT,WARMS4C_Y)
  OUTPUT.mixingregime[POLYMICTIC_CONT_WARM] <-2
  
  # 3 = POLYMICTIC_COLD (DISCONTINUOUS)
  POLYMICTIC_DISC_COLD  <- intersect(MIXING_MULT,WARMS4C_N)
  OUTPUT.mixingregime[POLYMICTIC_DISC_COLD] <-3
  
  # 4 = POLYMICTIC_WARM (DISCONTINUOUS)
  POLYMICTIC_DISC_WARM  <- intersect(MIXING_MULT,WARMS4C_Y)
  OUTPUT.mixingregime[POLYMICTIC_DISC_WARM] <-4
  
  # 5 = MONOMICTIC (COLD, MIX IN WINTER)
  MONOMICTIC_COLD  <- intersect(MIXING_ONCE,WARMS4C_N)
  OUTPUT.mixingregime[MONOMICTIC_COLD] <-5
  
  # 6 = MONOMICTIC (WARM, ICEFREE AND MISCING ONCE PER YEAR)
  MONOMICTIC_WARM  <- intersect(MIXING_ONCE,WARMS4C_Y)
  OUTPUT.mixingregime[MONOMICTIC_WARM] <-6
  
  # 7 = DIMICTIC   (MIX IN SUMMER AND WINTER)
  DIMICTIC         <- MIXING_TWICE
  OUTPUT.mixingregime[DIMICTIC] <-7
  
  # 8 = AMICTIC (ALWAYS ICE)
  AMICTIC           <- intersect(PERMANENT_STRATIFIED,ICECOVERED)
  OUTPUT.mixingregime[AMICTIC] <-8
  
  # 9 = MEROMICTIC(ICEFREE AND PERMANENTLY STRATIFIED) 
  MEROMICTIC      <- setdiff(PERMANENT_STRATIFIED,ICECOVERED)
  OUTPUT.mixingregime[MEROMICTIC] <-9
  
  check<-length(POLYMICTIC_CONT_COLD)+
    length(POLYMICTIC_CONT_WARM)+
    length(POLYMICTIC_DISC_COLD)+
    length(POLYMICTIC_DISC_WARM)+
    length(AMICTIC) +
    length(MONOMICTIC_COLD) +
    length(DIMICTIC) +
    length(MONOMICTIC_WARM)+
    length(MEROMICTIC)
  
  
  return(list(OUTPUT.mixingregime = OUTPUT.mixingregime,
              POLYMICTIC_CONT_COLD = POLYMICTIC_CONT_COLD,
              POLYMICTIC_CONT_WARM = POLYMICTIC_CONT_WARM,
              POLYMICTIC_DISC_COLD = POLYMICTIC_DISC_COLD,
              POLYMICTIC_DISC_WARM = POLYMICTIC_DISC_WARM,
              MONOMICTIC_COLD = MONOMICTIC_COLD,
              MONOMICTIC_WARM = MONOMICTIC_WARM,
              DIMICTIC = DIMICTIC,
              AMICTIC = AMICTIC, 
              MEROMICTIC = MEROMICTIC,
              check= check==NumberOfLakes))
}

#Legend functon##############################################################################################################
g_legend<-function(a.gplot){
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}


#statistic label function##############################################################################################################
generate_label_df <- function(TUKEY, variable){
  
  # Extract labels and factor levels from Tukey post-hoc 
  Tukey.levels <- TUKEY[[variable]][,4]
  Tukey.labels <- data.frame(multcompLetters(Tukey.levels)['Letters'])
  
  #I need to put the labels in the same order as in the boxplot :
  Tukey.labels$treatment=rownames(Tukey.labels)
  Tukey.labels=Tukey.labels[order(Tukey.labels$treatment) , ]
  return(Tukey.labels)
}
