#######################################
#Author      : Annette Janssen
#Institution : Wageningen University
#Date        : January 2021
#######################################

rm(list = ls())
#.libPaths("PATHLIBRARY")


# OPEN LIBRARIES
library(ncdf4)
library(abind)

# DEFINE FOLDERS
VICINFOLDER <- "./VICinput/"
VICOUTfolder <- "./VICoutput/"
OUTPUTFOLDER <- "./VICanalysis/"
FUNCTIONFOLDER <- "./Functions/"
filenames_in   <- list.files(VICINFOLDER,"VIC_params_hydrolake_AnnetteJanssen_global2.nc")

# DEFINE FILENAMES
filenames_hi <- list.files(VICOUTfolder,"historical.1861-01-01.nc")
filenames_pi <- list.files(VICOUTfolder,"piControl.1661-01-01.nc")

# OPEN FUNCTIONS
functions.file = paste(FUNCTIONFOLDER,"TemperatureFunctionsV4.R",sep = "")
source(functions.file)

#OPEN INPUT FILES###############################################################################################################

# OPEN INPUT FILES
nc.INPUT<-nc_open(paste(VICINFOLDER,"/",filenames_in,sep = ""), write=FALSE, readunlim=TRUE, verbose=FALSE,
                  auto_GMT=TRUE, suppress_dimvals=FALSE )

INPUT.lake_id   <- ncvar_get(nc = nc.INPUT,varid = "lake_id")
INPUT.source_id <- ncvar_get(nc = nc.INPUT,varid = "source_id")
INPUT.depth     <- ncvar_get(nc = nc.INPUT,varid = "depth_in")
INPUT.mask      <- ncvar_get(nc = nc.INPUT,varid = "depth_in")

# FIND LOCATIONS OF LAKES
idv                    <- which(INPUT.mask >0,arr.ind=TRUE)
ida                    <- data.frame(idv)
NumberOfLakes          <- length(ida$dim2)
node.depth             <- c(0.6, rep(0.5, 2000))

# MAKE DATA ARRAY OF INPUT VARIABLES
INPUTarray = data.frame(
  VICid          = INPUT.lake_id[idv],
  HLid           = INPUT.source_id[idv],
  LKdepth        = INPUT.depth[idv],
  lat            = nc.INPUT$dim$lat$vals[ida$dim2], 
  lon            = nc.INPUT$dim$lon$vals[ida$dim1], 
  x              = ida$dim1,
  y              = ida$dim2,
  z              = ida$dim3
)

saveRDS(INPUTarray, file = paste(OUTPUTFOLDER,"INPUTarray.csv",sep=""))

nc_close(nc.INPUT)

#OPEN HISTORICAL FILES###############################################################################################################

# OPEN HISTORIC FILES
for (filename in filenames_hi[4]){
  nc.OUTPUT               <- nc_open(paste(VICOUTfolder,"/",filename,sep = ""), write=FALSE, readunlim=TRUE, verbose=FALSE,
                                     auto_GMT=TRUE, suppress_dimvals=FALSE )
  model.name              <- gsub("fluxes_hydrolake_AnnetteJanssen_global_","",gsub("_historical.1861-01-01.nc","",filename))
  
  # GET OUTPUT
  OUTPUT.dates            <- as.Date(nc.OUTPUT$dim$time$vals, origin = "0000-12-30")
  OUTPUT.months           <- as.numeric(format.Date(OUTPUT.dates, "%m"))
  OUTPUT.years            <- as.numeric(format.Date(OUTPUT.dates, "%Y"))
  OUTPUT.yearsUnique      <- unique(OUTPUT.years)
  
  # PREPARE OUTPUT ARRAY PER LAKE
  OUTPUT.mixingregime     <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.T.epi.averyr     <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.T.hyp.averyr     <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.T.surf.averyr    <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.str.depth.averyr <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.str.averyr       <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.ice.days         <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  OUTPUT.albedo           <- array(0,c(NumberOfLakes,length(OUTPUT.yearsUnique)))
  
  for (yr in OUTPUT.yearsUnique){  
    #nc.OUTPUT               <- nc_open(paste(VICOUTfolder,"/",filename,sep = ""), write=FALSE, readunlim=TRUE, verbose=FALSE)
    print(paste("Year", yr))
    Year.start  <- min(which(OUTPUT.years ==yr))
    Year.end    <- max(which(OUTPUT.years ==yr))
    Year.days   <- Year.end-Year.start+1
    
    # PREPARE OUTPUT ARRAY PER YEAR
    OUTPUT.T.epi           <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.T.hyp           <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.T.surf          <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.T.ice           <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.str             <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.str.depth       <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.depth           <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.evaporation     <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.mLOut           <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.albedo          <- array(0,c(NumberOfLakes,Year.days)) 
    OUTPUT.prec            <- array(0,c(NumberOfLakes,Year.days)) 
    
    for(i in c(Year.start:Year.end)){
      print(paste("Day", i, "of",Year.days))
      
      # CALCULATE TEMPERATURE VARIABLES
      temp.vars = calculateTemperatureVariables(nc.OUTPUT)
      OUTPUT.T.epi[,i-Year.start+1]           <- temp.vars$water.temp.epi
      OUTPUT.T.hyp[,i-Year.start+1]           <- temp.vars$water.temp.hyp
      OUTPUT.T.surf[,i-Year.start+1]          <- temp.vars$water.temp.surf
      OUTPUT.T.ice[,i-Year.start+1]           <- temp.vars$ice.temp
      OUTPUT.str[,i-Year.start+1]             <- temp.vars$strat
      OUTPUT.str.depth[,i-Year.start+1]       <- temp.vars$thermo.depth
      OUTPUT.depth[,i-Year.start+1]           <- temp.vars$waterdepth
      
      # CALCULATE ENERGY VARIABLES
      eb.vars = calculateEnergyVariables(nc.OUTPUT)
      OUTPUT.evaporation[,i-Year.start+1]     <- eb.vars$evaporation
      OUTPUT.mLOut[,i-Year.start+1]           <- eb.vars$sw.down
      OUTPUT.albedo[,i-Year.start+1]          <- eb.vars$albedo
      OUTPUT.prec[,i-Year.start+1]            <- eb.vars$precipitation
    }
    
    # WRITE DAILY DATA IN CSV FILE FOR EACH YEAR
    write.csv(round(OUTPUT.T.epi,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_Tepi.csv",sep=""))
    write.csv(round(OUTPUT.T.hyp,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_Thyp.csv",sep=""))
    write.csv(round(OUTPUT.T.surf,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_TSurf.csv",sep=""))
    write.csv(round(OUTPUT.str,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_stratification.csv",sep=""))
    write.csv(round(OUTPUT.str.depth,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_strdep.csv",sep=""))
    write.csv(round(OUTPUT.depth,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_waterdepth.csv",sep=""))
    write.csv(round(OUTPUT.evaporation,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_evap.csv",sep=""))
    write.csv(round(OUTPUT.mLOut,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_mLOut.csv",sep=""))
    write.csv(round(OUTPUT.albedo,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_albedo.csv",sep=""))
    write.csv(round(OUTPUT.prec,digits = 2),paste(OUTPUTFOLDER,model.name,"_" ,yr,"_prec.csv",sep=""))
    
    # CALCULATE STRATIFICATION TYPE
    IceDays         <- round(rowSums(OUTPUT.T.ice<0)/Year.days,2)
    IceThreshold    <- 0.95
    StratThreshold  <- 30
    daysInYear      <- Year.days
    periodSmooth    <- 5
    meroThreshold   <- 365*0.01
    
    OUTPUT.StratType <- calculateStratficationType(IceDays,
                                                   IceThreshold,
                                                   OUTPUT.str,
                                                   StratThreshold,
                                                   OUTPUT.T.epi,
                                                   Year.days,
                                                   periodSmooth,
                                                   meroThreshold)
    
    
    # AVERAGE OUTPUT PER YEAR
    
    OUTPUT.mixingregime[,which(OUTPUT.yearsUnique==yr)]        <- OUTPUT.StratType$OUTPUT.mixingregime
    OUTPUT.T.epi.averyr[,which(OUTPUT.yearsUnique==yr)]        <- round(rowMeans(OUTPUT.T.epi),2)
    OUTPUT.T.hyp.averyr[,which(OUTPUT.yearsUnique==yr)]        <- round(rowMeans(OUTPUT.T.hyp),2)
    OUTPUT.T.surf.averyr[,which(OUTPUT.yearsUnique==yr)]       <- round(rowMeans(OUTPUT.T.surf),2)
    OUTPUT.str.depth.averyr[,which(OUTPUT.yearsUnique==yr)]    <- round(rowMeans(OUTPUT.str.depth),2)
    OUTPUT.str.averyr[,which(OUTPUT.yearsUnique==yr)]          <- round(rowSums(OUTPUT.str>0)/Year.days,2)
    OUTPUT.ice.days[,which(OUTPUT.yearsUnique==yr)]            <- IceDays
    OUTPUT.albedo[,which(OUTPUT.yearsUnique==yr)]              <- round(rowSums(OUTPUT.albedo)/Year.days,2) 
    
  } 
  
  
  # WRITE YEARLY DATA IN CSV FILE FOR EACH CLIMATE MODEL
  write.csv(OUTPUT.mixingregime,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.StratType.csv",sep=""))
  write.csv(OUTPUT.T.epi.averyr,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.T.epi.averyr.csv",sep=""))
  write.csv(OUTPUT.T.hyp.averyr,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.T.hyp.averyr.csv",sep=""))
  write.csv(OUTPUT.T.surf.averyr,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.T.surf.averyr.csv",sep=""))
  write.csv(OUTPUT.str.depth.averyr,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.str.depth.averyr.csv",sep=""))
  write.csv(OUTPUT.str.averyr,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.str.avery.csv",sep=""))
  write.csv(OUTPUT.ice.days,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.ice.days.csv",sep=""))
  write.csv(OUTPUT.albedo,paste(OUTPUTFOLDER,model.name,"_" ,"_OUTPUT.albedo.csv",sep=""))
  
  nc_close(nc.OUTPUT)
}



