#######################################
#Author      : Annette Janssen
#Institution : Wageningen University
#Date        : June 2021
#######################################

rm(list = ls())
#.libPaths("PATHLIBRARY")

# OPEN LIBRARIES
library(greybox)
library(smooth)
library(ggplot2)
library(gridExtra)
library(DescTools)
library(rgdal)
library(matrixStats)
library(ggrepel)
library(abind)
library(patchwork)
library(gridExtra)
library(cowplot)
library(ggpubr)
library(multcompView)
require(scales) # to access break formatting functions
library(coin)
library(survival)
library(rcompanion)
library(colorspace)

# DEFINE FOLDERS
WORRKINGDIRECTORY <- "MYWORKINGDIRECTORY"
setwd(WORRKINGDIRECTORY)
INPUTFOLDER       <- "./InputData"
OUTPUTFOLDER      <- "./OutputData"
ANALYSISFOLDER    <- "./AnalysisData"
FIGURESFOLDER     <- "./Figures"
OUTPUTFILES       <- list.files(OUTPUTFOLDER)
MODELS            <- unique(substr(OUTPUTFILES,0,regexpr('__OUTPUT', OUTPUTFILES)-1))[c(1,2,3,4)]
FUNCTIONFOLDER    <- "./Functions/"
functions.file = paste(FUNCTIONFOLDER,"Functions.R",sep = "")
source(functions.file)

# fOLDERS FOR BIGDATA
BIGDATAFOLDER     <- "./MYDATAFOLDER"

# SETTINGS
strat_clas_period <- c(1900:2000)
StratThreshold<-30
IceThreshold <- 0.95
daysInYear<-365
periodSmooth<-5
meroThreshold   <- 365*0.01

# INPUT
lakeType <-c("Polymictic continuous (cold)",
             "Polymictic continuous (warm)",
             "Polymictic discontinuous (cold)",
             "Polymictic discontinuous (warm)",
             "Amictic",             
             "Monomictic cold",
             "Dimictic",            
             "Monomictic warm",
             "Meromictic")
lakeType_function <- lakeType[c(1,2,3,4,6,8,7,5,9)]
INPUTarray        <- readRDS(paste(INPUTFOLDER,"/INPUTarray.csv",sep=""))
NumberOfLakes     <- length(INPUTarray[,1])

################## Caclculations ##############################################################

# DOWNLOAD SHAPEFILE
HydroLAKESandSOIL<- readOGR(dsn= paste0("myFolder"), 
                            layer="HydroLAKES_polys_v10_CN_soil_Qlt0",
                            verbose=FALSE)

# DFIND CORROSPONDING ID BETWEEN DATA INPUT FILES AND DATA OUTPUT FILES
ID                <- vector('numeric',length(INPUTarray[,1]))
for(i in 1:length(INPUTarray[,1])){
  ID[i] <- which(HydroLAKESandSOIL$Hylak_id==INPUTarray$HLid[i])[1]
}

# MAKE DATA FILE OF INPUTDATA
DATAtable <-data.frame(InputData  <- INPUTarray,
                       Hylak_id   <- HydroLAKESandSOIL$Hylak_id[ID],
                       LakeName   <- HydroLAKESandSOIL$Lake_name[ID],
                       LakeType   <- HydroLAKESandSOIL$Lake_type[ID],
                       latHL      <- HydroLAKESandSOIL$Pour_lat[ID],
                       lonHL      <- HydroLAKESandSOIL$Pour_long[ID],
                       depthHL    <- HydroLAKESandSOIL$Depth_avg[ID],
                       area       <- HydroLAKESandSOIL$Lake_area[ID],
                       rel_depth  <- HydroLAKESandSOIL$Depth_avg[ID]/(sqrt(HydroLAKESandSOIL$Lake_area[ID])*1000),
                       RT         <- HydroLAKESandSOIL$Res_time[ID],
                       Dis_avg    <- HydroLAKESandSOIL$Dis_avg[ID],
                       elevation  <- as.numeric(HydroLAKESandSOIL$Elevation[ID]),
                       volume     <- as.numeric(HydroLAKESandSOIL$Vol_total[ID]),
                       shoredev   <- as.numeric(HydroLAKESandSOIL$Shore_dev[ID]),
                       clay_S     <- HydroLAKESandSOIL$T_CLAY[ID],
                       clay_T     <- HydroLAKESandSOIL$S_CLAY[ID],
                       silt_S     <- HydroLAKESandSOIL$T_SILT[ID],
                       silt_T     <- HydroLAKESandSOIL$S_SILT[ID],
                       S_OC       <- HydroLAKESandSOIL$T_OC[ID],
                       T_OC       <- HydroLAKESandSOIL$S_OC[ID],
                       sand_S     <- HydroLAKESandSOIL$T_SAND[ID],
                       sand_T     <- HydroLAKESandSOIL$S_SAND[ID],
                       gravel_T   <- HydroLAKESandSOIL$T_GRAVEL[ID],
                       bulk_den_T <- HydroLAKESandSOIL$T_BULK_DEN[ID],
                       bulk_ref_T <- HydroLAKESandSOIL$T_REF_BULK[ID],
                       ustex      <- HydroLAKESandSOIL$S_USDA_TEX[ID])

#saveRDS(DATAtable,paste(ANALYSISFOLDER,"/DATAtable.csv",sep=""))
#DATAtable<-readRDS(paste(ANALYSISFOLDER,"/DATAtable.csv",sep=""))

# CALCULATE STRATIFICATION REGIME BASED ON AVERAGE TEMPERATURE IN HYP AND EPI
val_ave           <- 1/(length(strat_clas_period)*length(MODELS)) # change to 4
Tepi              <- matrix(0,ncol = 365,nrow= NumberOfLakes)
Thyp              <- matrix(0,ncol = 365,nrow= NumberOfLakes)
TSurf             <- matrix(0,ncol = 365,nrow= NumberOfLakes)
Evap             <- matrix(0,ncol = 365,nrow= NumberOfLakes)
Prec             <- matrix(0,ncol = 365,nrow= NumberOfLakes)
IceDays           <- matrix(0,ncol = 1 ,nrow= NumberOfLakes)
Albedo           <- matrix(0,ncol = 1 ,nrow= NumberOfLakes)
Stratdepth       <-matrix(0,ncol = 1 ,nrow= NumberOfLakes)
str              <-matrix(0,ncol = 1 ,nrow= NumberOfLakes)

for (aModel in MODELS){
  aModel=MODELS[1]
  
  IceDays_dum<- read.csv(paste(OUTPUTFOLDER,"/",aModel,"__OUTPUT.ice.days.csv",sep="",dec=""),row.names = 1)
  IceDays_dum <- rowMeans(IceDays_dum)
  IceDays <- IceDays_dum/length(MODELS)+IceDays
  
  print(aModel)
  for (yr in strat_clas_period) {
    print(yr)
    Tepi_dum  <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_Tepi.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    Thyp_dum  <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_Thyp.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    TSurf_dum <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_TSurf.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    Evap_dum  <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_evap.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    Prec_dum  <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_prec.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    Albedo_dum  <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_albedo.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    Ice_dum     <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_ice.days.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    strdepth_dum     <- read.csv(paste(BIGDATAFOLDER,"/",aModel,"/",aModel,"_",yr,"_strdep.csv",sep="",dec=""),row.names = 1)[,1:365] #ommitting leapyear day 366
    
    Tepi  <- Tepi_dum * val_ave +Tepi
    Thyp  <- Thyp_dum * val_ave +Thyp
    TSurf <- TSurf_dum * val_ave +TSurf
    Evap  <- Evap_dum* val_ave +Evap
    Prec  <- Prec_dum* val_ave +Prec
    Albedo  <- Albedo_dum* val_ave +Albedo
    
  }
}



# saveRDS(Tepi,paste(ANALYSISFOLDER,"/Tepi_ave.csv",sep=""))
# saveRDS(Thyp,paste(ANALYSISFOLDER,"/Thyp_ave.csv",sep=""))
# saveRDS(IceDays,paste(ANALYSISFOLDER,"/IceDays.csv",sep=""))
# saveRDS(TSurf,paste(ANALYSISFOLDER,"/TSurf.csv",sep=""))
# saveRDS(Evap,paste(ANALYSISFOLDER,"/Evap.csv",sep=""))
# saveRDS(Prec,paste(ANALYSISFOLDER,"/Prec.csv",sep=""))
# saveRDS(Albedo,paste(ANALYSISFOLDER,"/Albedo.csv",sep=""))
# saveRDS(Stratdepth,paste(ANALYSISFOLDER,"/Stratdepth.csv",sep=""))
# saveRDS(str,paste(ANALYSISFOLDER,"/str.csv",sep=""))

Tepi  <-readRDS(paste(ANALYSISFOLDER,"/Tepi_ave.csv",sep=""))
Thyp  <-readRDS(paste(ANALYSISFOLDER,"/Thyp_ave.csv",sep=""))
IceDays<-readRDS(paste(ANALYSISFOLDER,"/IceDays.csv",sep=""))
TSurf<-readRDS(paste(ANALYSISFOLDER,"/TSurf.csv",sep=""))
Evap<-readRDS(paste(ANALYSISFOLDER,"/Evap.csv",sep=""))
Prec<-readRDS(paste(ANALYSISFOLDER,"/Prec.csv",sep=""))
Albedo<-readRDS(paste(ANALYSISFOLDER,"/Albedo.csv",sep=""))
Stratdepth<-readRDS(paste(ANALYSISFOLDER,"/Stratdepth.csv",sep=""))


density_Tepi = 999.842594 + 
  (6.793952 * 10^-2 * Tepi) - 
  (9.095290 * 10^-3 * Tepi ^ 2) + 
  (1.001685 * 10^-4 * Tepi ^ 3) - 
  (1.120083 * 10^-6 * Tepi ^ 4) +
  (6.536336 * 10^-9 * Tepi ^ 5) # Based on ISIMIP protocol

density_Thyp = 999.842594 + 
  (6.793952 * 10^-2 * Thyp) - 
  (9.095290 * 10^-3 * Thyp ^ 2) + 
  (1.001685 * 10^-4 * Thyp ^ 3) - 
  (1.120083 * 10^-6 * Thyp ^ 4) +
  (6.536336 * 10^-9 * Thyp ^ 5) # Based on ISIMIP protocol


Strat<- data.frame(array(as.numeric(abs(density_Tepi - density_Thyp) > 0.1),c(NumberOfLakes,365)))
THyp_row <- which(rowMeans(Thyp)<=0)
Strat[THyp_row,]<-0


StratClass<-calculateStratficationType(IceDays,IceThreshold, Strat,StratThreshold,Tepi,TSurf,daysInYear,periodSmooth,meroThreshold)
STRperiod<-StratClass$strperiod


sum(which(StratClass$OUTPUT.mixingregime==1)>0)
sum(which(StratClass$OUTPUT.mixingregime==2)>0)
sum(which(StratClass$OUTPUT.mixingregime==3)>0)
sum(which(StratClass$OUTPUT.mixingregime==4)>0)
sum(which(StratClass$OUTPUT.mixingregime==5)>0)
sum(which(StratClass$OUTPUT.mixingregime==6)>0)
sum(which(StratClass$OUTPUT.mixingregime==7)>0)
sum(which(StratClass$OUTPUT.mixingregime==8)>0)
sum(which(StratClass$OUTPUT.mixingregime==9)>0)


# CALCULATE AVERAGE STRATIFICATION REGIME BASED ON MOST COMMON REGIME OVER 100 YEARS OF YEARLY EPI AND HYP TEMPERATURES 
simyears <- strat_clas_period
strt <- which(simyears == min(strat_clas_period))+1
nd<- which(simyears == max(strat_clas_period))+1
S_strat_common      <- matrix(0,ncol = 4 ,nrow= NumberOfLakes)
S_strat_common_perc <- matrix(0,ncol = 4 ,nrow= NumberOfLakes)
uniqx<-c(1:9)
for (aModel in MODELS){
  S_data<-read.csv(paste(OUTPUTFOLDER,'/',aModel,'__OUTPUT.StratType.csv',sep=''))
  S_data_sub<-S_data[,c(strt:nd)]
  for( i in 1:NumberOfLakes){
    S_strat_common[i,which(MODELS==aModel)] <-uniqx[which.max(tabulate(match(S_data_sub[i,], uniqx)))]
    S_strat_common_perc[i,which(MODELS==aModel)]<-max(tabulate(match(S_data_sub[i,], uniqx)))/(nd-strt+1)
  }
}

#saveRDS(S_strat_common,paste(ANALYSISFOLDER,"/S_strat_common.csv",sep=""))
#saveRDS(S_strat_common_perc,paste(ANALYSISFOLDER,"/S_strat_common_perc.csv",sep=""))

# calculate water balance for salt lakes
Evapm3  <- (rowSums(Evap)[DATAtable$VICid+1]/1000)*(DATAtable$area*1000000)
Qm3     <- DATAtable$Dis_avg....HydroLAKESandSOIL.Dis_avg.ID.
Pm3     <- (rowSums(Prec)[DATAtable$VICid+1]/1000)*(DATAtable$area*1000000)
EtoQ    <- Evapm3/(Qm3-Evapm3+Pm3)


DATAtable2 <- data.frame(DATAtable,
                         T_Surf<-rowMeans(TSurf)[DATAtable$VICid+1],
                         T_Surf_max<-apply(TSurf, 1, max)[DATAtable$VICid+1] ,
                         T_Surf_min<-apply(TSurf, 1, min)[DATAtable$VICid+1] ,
                         T_Epi<-rowMeans(Tepi)[DATAtable$VICid+1],
                         T_Hyp<-rowMeans(Thyp)[DATAtable$VICid+1],
                         T_Epi_max<-apply(Tepi, 1, max)[DATAtable$VICid+1] ,
                         T_Hyp_max<-apply(Thyp, 1, max)[DATAtable$VICid+1] ,
                         T_Epi_min<-apply(Tepi, 1, min)[DATAtable$VICid+1] ,
                         T_Hyp_min<-apply(Thyp, 1, min)[DATAtable$VICid+1] ,
                         Albedos<-apply(Albedo, 1, mean)[DATAtable$VICid+1] ,
                         Evaps<-rowSums(Evap)[DATAtable$VICid+1],
                         Precs<-rowSums(Prec)[DATAtable$VICid+1],
                         NrIceDays<-IceDays[DATAtable$VICid+1]*365,
                         EtoQ<- EtoQ,
                         Stratification<-StratClass$OUTPUT.mixingregime[DATAtable$VICid+1],
                         StratName<-lakeType_function[StratClass$OUTPUT.mixingregime[DATAtable$VICid+1]],
                         Stratdepths<- rowMeans(Stratdepth)[DATAtable$VICid+1],
                         S_strat_common[DATAtable$VICid+1,],
                         S_strat_common_perc[DATAtable$VICid+1,])


S_strat_common1      <- S_strat_common[DATAtable$VICid+1,]
S_strat_common_perc1 <- S_strat_common_perc[DATAtable$VICid+1,]

colnames(DATAtable2)<-c("VICid","HLid","LKdepth","lat","lon" ,"x" , "y" , "z" ,"Hylak_id","LakeName",                                           
                        "LakeType","latHL",                                               
                        "lonHL", "depthHL",                                            
                        "area",  "rel_depth",
                        "RT","Dis_avg",
                        "elevation",
                        "volume","shore_dev",
                        "clay_S",  "clay_T",
                        "silt_S",  "silt_T",
                        "OC_S",  "OC_T",
                        "sand_S",  "sand_T",
                        "gravel_T","bulk_T","bulk_ref_T","ustex",
                        "TSurf","TSurfMax","TSurfMin","TEpi","THyp","TEpiMax","THypMax","TEpiMin","THypMin",
                        "Albedo",
                        "Evap",'Prec',"IceDays",
                        'EvaptoQin',
                        "Stratification","lakeTypeName","StratDepth",
                        paste(MODELS,'_strat_method2',sep=""),paste(MODELS,'_strat_perc_method2',sep=""))

DATAtablePCLAKE_T_Surf<-data.frame(DATAtable$VICid,
                                   Stratification<-StratClass$OUTPUT.mixingregime[DATAtable$VICid+1],
                                   TSurf[DATAtable$VICid+1,])
DATAtablePCLAKE_T_Epi<-data.frame(DATAtable$VICid,
                                  Stratification<-StratClass$OUTPUT.mixingregime[DATAtable$VICid+1],
                                  T_epi<-Tepi[DATAtable$VICid+1,])
DATAtablePCLAKE_T_Hyp<-data.frame(DATAtable$VICid,
                                  Stratification<-StratClass$OUTPUT.mixingregime[DATAtable$VICid+1],
                                  T_hyp<-Thyp[DATAtable$VICid+1,])
DATAtableStratClass <-data.frame(DATAtable$VICid,
                                 Stratification<-StratClass$OUTPUT.mixingregime[DATAtable$VICid+1],
                                 StratCl<-STRperiod[DATAtable$VICid+1,])


colnames(DATAtablePCLAKE_T_Surf)<-c("VICid","Stratification")
colnames(DATAtablePCLAKE_T_Epi)<-c("VICid","Stratification")
colnames(DATAtablePCLAKE_T_Hyp)<-c("VICid","Stratification")
colnames(DATAtableStratClass)<-c("VICid","Stratification")

#saveRDS(DATAtable2,paste(ANALYSISFOLDER,"/DATAtable2.csv",sep=""))
#DATAtable2<-readRDS(paste(ANALYSISFOLDER,"/DATAtable2.csv",sep=""))

# calculate adjusted latitude

Lewis1983<-array(c(c(0:9)*10,0.27,0.31,0.34,0.39,0.46,0.54,0.68,0.89,1.3,2.4),dim=(c(10,2)))
Lewis1983_lin<-array(unlist(approx(x=Lewis1983[,1],y=Lewis1983[,2],n=100)),dim=(c(100,2)))

id_latadjust<-DATAtable2$latHL[i]
for(i in 1:length(DATAtable2[,1])){
  id_latadjust[i]<-which(abs(Lewis1983_lin[,1]-DATAtable2$latHL[i])==min(abs(Lewis1983_lin[,1]-DATAtable2$latHL[i])))
}

# add adjusted latitude to dataframe
DATAtable2a<-data.frame(DATAtable2,latadj<-DATAtable2$latHL+DATAtable2$elevation/100*Lewis1983_lin[id_latadjust,2])



colnames(DATAtable2a)<-c(colnames(DATAtable2),"LatAdj")

#exclude lakes because they are always frozen (I=Ice)
Exl_I<-which(DATAtable2a$TSurf<=0)

#exclude lakes because of no sediment data (SED=sediment, BD= Bulk density)
Exl_SED<-which((DATAtable2a$clay_T+DATAtable2a$silt_T+DATAtable2a$sand_T)==0)
Exl_BD<-which((DATAtable2a$bulk_ref_T)==0)

#exclude lakes because the are salt (ST=salt)
vicid=c(1,28,40,53,63,65,86,87,70,73,106) #lakes that are for sure salt
ID=c(1:length(vicid))*0
for (i in 1:length(vicid)){
  ID[i]<-which(DATAtable2a$VICid==vicid[i])
}
Exl_ST <- unique(c(which(DATAtable2a$EvaptoQin>1),ID))

Exl_lakes <-unique(c(Exl_I,Exl_SED,Exl_BD,Exl_ST))
Inl_lakes <-setdiff(c(1:length(DATAtable2a$VICid)),Exl_lakes)

DATAtable3<-DATAtable2a[Inl_lakes,]
DATAtablePCLAKE_T_Surf1<-DATAtablePCLAKE_T_Surf[Inl_lakes,]
DATAtablePCLAKE_T_Epi1<-DATAtablePCLAKE_T_Epi[Inl_lakes,]
DATAtablePCLAKE_T_Hyp1<-DATAtablePCLAKE_T_Hyp[Inl_lakes,]
DATAtableStratClass1<-DATAtableStratClass[Inl_lakes,]

#saveRDS(DATAtable3,paste(ANALYSISFOLDER,"/DATAtable3.csv",sep=""))
write.csv(DATAtable3,paste(ANALYSISFOLDER,"/DATAtable_ARCGIS.csv",sep=""))

# saveRDS(DATAtablePCLAKE_T_Surf1,paste(ANALYSISFOLDER,"/DATAtablePCLAKE_T_Surf.csv",sep=""))
# saveRDS(DATAtablePCLAKE_T_Epi1,paste(ANALYSISFOLDER,"/DATAtablePCLAKE_T_Epi.csv",sep=""))
# saveRDS(DATAtablePCLAKE_T_Hyp1,paste(ANALYSISFOLDER,"/DATAtablePCLAKE_T_Hyp.csv",sep=""))
# saveRDS(DATAtableStratClass1,paste(ANALYSISFOLDER,"/DATAtableStratClass.csv",sep=""))

S_strat_common_incl2<-S_strat_common1[Inl_lakes,]
S_strat_common_perc_incl2<-S_strat_common_perc1[Inl_lakes,]

colnames(S_strat_common_incl2)<-MODELS
colnames(S_strat_common_perc_incl2)<-MODELS

#saveRDS(S_strat_common_incl2,paste(ANALYSISFOLDER,"/S_strat_common_incl.csv",sep=""))
#saveRDS(S_strat_common_perc_incl2,paste(ANALYSISFOLDER,"/S_strat_common_perc_incl.csv",sep=""))



################## FIGUREs loading data ##############################################################
DATA<- readRDS(paste(ANALYSISFOLDER,"/DATAtable3.csv",sep=""))
clmap <- c("ivory4","yellow2","red","turquoise","royalblue","orchid","olivedrab","orange","lightpink")
clmap_light <- c("ivory4","yellow2","red","turquoise","royalblue","orchid","olivedrab","orange","lightpink")
#clmap <- c("royalblue","olivedrab","orchid","lightpink", "ivory4","turquoise","yellow2","orange","red")


################## FIGURE 1 ##############################################################
# figure made in ARCGIS

################## FIGURE 2 ##############################################################
# figure made in validation script

################## FIGURE 3 ############################################################## 
newStrat<-DATA$Stratification
newStrat[which(DATA$Stratification==1)]<- 1
newStrat[which(DATA$Stratification==2)]<- 2
newStrat[which(DATA$Stratification==3)]<- 3
newStrat[which(DATA$Stratification==4)]<- 4
newStrat[which(DATA$Stratification==5)]<- 6
newStrat[which(DATA$Stratification==6)]<- 8
newStrat[which(DATA$Stratification==7)]<- 7
newStrat[which(DATA$Stratification==8)]<- 5
newStrat[which(DATA$Stratification==9)]<- 9

DATA_dum<-data.frame(DATA,newStrat)

fig3lab <- array(0,c(7,1))
fig3lab[1]<- sum(which(DATA_dum$newStrat==1)>0)
fig3lab[2]<- sum(which(DATA_dum$newStrat==2)>0)
fig3lab[3]<- sum(which(DATA_dum$newStrat==3)>0)
fig3lab[4]<- sum(which(DATA_dum$newStrat==4)>0)
fig3lab[5]<- sum(which(DATA_dum$newStrat==5)>0)
fig3lab[6]<- sum(which(DATA_dum$newStrat==6)>0)
fig3lab[7]<- sum(which(DATA_dum$newStrat==7)>0)
fig3lab[8]<- sum(which(DATA_dum$newStrat==8)>0)
fig3lab[9]<- sum(which(DATA_dum$newStrat==9)>0)

#FIGURE 3a
figure3a<-ggplot() + 
  geom_bar(data = DATA_dum,aes(x=newStrat ,fill=DATA$lakeTypeName)) +
  scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                labels = trans_format("log10", math_format(10^.x)))   +
  scale_x_continuous(breaks =c(1,2,3,4,5,6,7,8,9) ,
                     labels = lakeType)+
  theme(panel.background = element_rect(fill = "white",
                                        colour = "white",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "none",
        axis.text.x = element_text(angle=0, size=16,hjust=1),
        axis.text.y = element_text(size=16),
        axis.title =element_text(size=16))+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1,2,3,4,5,6,7,8,9)])+
  geom_text(aes(x=c(1,2,3,4,5,6,7,8,9),y=c(fig3lab*2)),label=fig3lab,size=5)+
  ylab("Number of lakes (-)")+ 
  xlab("")+
  coord_flip()


#FIGURE 3b
HuHuanyongLine <-data.frame(array(c(c(24.431742, 50.275053),c(97.529543, 127.584593)),dim=(c(2,2))))
colnames(HuHuanyongLine)<-c("lat","lon")
formLine<-lm(lat~lon,HuHuanyongLine)
EASTlakes<-((formLine$coefficients[1]+formLine$coefficients[2]*DATA_dum$lon)>DATA_dum$lat)
WESTlakes<-((formLine$coefficients[1]+formLine$coefficients[2]*DATA_dum$lon)<=DATA_dum$lat)

sum(EASTlakes>0)/length(EASTlakes)*100
sum(WESTlakes>0)/length(EASTlakes)*100



data_fraction_1 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA[EASTlakes,]$Stratification==1)/sum(DATA$Stratification==1),
                                           sum(DATA[WESTlakes,]$Stratification==1)/sum(DATA$Stratification==1))))
data_fraction_1$ymax = cumsum(data_fraction_1$fract)
data_fraction_1$ymin = c(0, head(data_fraction_1$ymax, n=-1))
data_fraction_1$label <- paste0(data_fraction_1$category, " (", round(data_fraction_1$fract*100,0),"%) ")
data_fraction_1$labelPosition <- (data_fraction_1$ymax + data_fraction_1$ymin) / 2
figure3b_1 <- ggplot(data_fraction_1, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[5]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[5],lighten(clmap[5], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_2 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==2)/sum(DATA$Stratification==2),
                                      sum(DATA[WESTlakes,]$Stratification==2)/sum(DATA$Stratification==2)))
data_fraction_2$ymax = cumsum(data_fraction_2$fract)
data_fraction_2$ymin = c(0, head(data_fraction_2$ymax, n=-1))
data_fraction_2$label <- paste0(data_fraction_2$category, " (", round(data_fraction_2$fract*100,0),"%) ")
data_fraction_2$labelPosition <- (data_fraction_2$ymax + data_fraction_2$ymin) / 2
figure3b_2 <-ggplot(data_fraction_2, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[7]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[7],lighten(clmap[7], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_3 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==3)/sum(DATA$Stratification==3),
                                      sum(DATA[WESTlakes,]$Stratification==3)/sum(DATA$Stratification==3)))
data_fraction_3$ymax = cumsum(data_fraction_3$fract)
data_fraction_3$ymin = c(0, head(data_fraction_3$ymax, n=-1))
data_fraction_3$label <- paste0(data_fraction_3$category, " (", round(data_fraction_3$fract*100,0),"%) ")
data_fraction_3$labelPosition <- (data_fraction_3$ymax + data_fraction_3$ymin) / 2
figure3b_3 <- ggplot(data_fraction_3, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[6]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[6],lighten(clmap[6], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_4 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==4)/sum(DATA$Stratification==4),
                                      sum(DATA[WESTlakes,]$Stratification==4)/sum(DATA$Stratification==4)))
data_fraction_4$ymax = cumsum(data_fraction_4$fract)
data_fraction_4$ymin = c(0, head(data_fraction_4$ymax, n=-1))
data_fraction_4$label <- paste0(data_fraction_4$category, " (", round(data_fraction_4$fract*100,0),"%) ")
data_fraction_4$labelPosition <- (data_fraction_4$ymax + data_fraction_4$ymin) / 2
figure3b_4 <- ggplot(data_fraction_4, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[9]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[9],lighten(clmap[9], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_5 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==8)/sum(DATA$Stratification==8),
                                      sum(DATA[WESTlakes,]$Stratification==8)/sum(DATA$Stratification==8)))
data_fraction_5$ymax = cumsum(data_fraction_5$fract)
data_fraction_5$ymin = c(0, head(data_fraction_5$ymax, n=-1))
data_fraction_5$label <- paste0(data_fraction_5$category, " (", round(data_fraction_5$fract*100,0),"%) ")
data_fraction_5$labelPosition <- (data_fraction_5$ymax + data_fraction_5$ymin) / 2
figure3b_5 <- ggplot(data_fraction_5, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[1]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[1],lighten(clmap[1], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_6 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==5)/sum(DATA$Stratification==5),
                                      sum(DATA[WESTlakes,]$Stratification==5)/sum(DATA$Stratification==5)))
data_fraction_6$ymax = cumsum(data_fraction_6$fract)
data_fraction_6$ymin = c(0, head(data_fraction_6$ymax, n=-1))
data_fraction_6$label <- paste0(data_fraction_6$category, " (", round(data_fraction_6$fract*100,0),"%) ")
data_fraction_6$labelPosition <- (data_fraction_6$ymax + data_fraction_6$ymin) / 2
figure3b_6 <- ggplot(data_fraction_6, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[4]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[4],lighten(clmap[4], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_7 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==7)/sum(DATA$Stratification==7),
                                      sum(DATA[WESTlakes,]$Stratification==7)/sum(DATA$Stratification==7)))
data_fraction_7$ymax = cumsum(data_fraction_7$fract)
data_fraction_7$ymin = c(0, head(data_fraction_7$ymax, n=-1))
data_fraction_7$label <- paste0(data_fraction_7$category, " (", round(data_fraction_7$fract*100,0),"%) ")
data_fraction_7$labelPosition <- (data_fraction_7$ymax + data_fraction_7$ymin) / 2
figure3b_7 <- ggplot(data_fraction_7, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[2]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[2],lighten(clmap[2], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_8 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==6)/sum(DATA$Stratification==6),
                                      sum(DATA[WESTlakes,]$Stratification==6)/sum(DATA$Stratification==6)))
data_fraction_8$ymax = cumsum(data_fraction_8$fract)
data_fraction_8$ymin = c(0, head(data_fraction_8$ymax, n=-1))
data_fraction_8$label <- paste0(data_fraction_8$category, " (", round(data_fraction_8$fract*100,0),"%) ")
data_fraction_8$labelPosition <- (data_fraction_8$ymax + data_fraction_8$ymin) / 2
figure3b_8 <- ggplot(data_fraction_8, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[8]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[8],lighten(clmap[8], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_9 <- data.frame(category=c("East","West"),
                              fract=c(sum(DATA[EASTlakes,]$Stratification==9)/sum(DATA$Stratification==9),
                                      sum(DATA[WESTlakes,]$Stratification==9)/sum(DATA$Stratification==9)))
data_fraction_9$ymax = cumsum(data_fraction_9$fract)
data_fraction_9$ymin = c(0, head(data_fraction_9$ymax, n=-1))
data_fraction_9$label <- paste0(data_fraction_9$category, " (", round(data_fraction_9$fract*100,0),"%) ")
data_fraction_9$labelPosition <- (data_fraction_9$ymax + data_fraction_9$ymin) / 2
figure3b_9 <- ggplot(data_fraction_9, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[3]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[3],lighten(clmap[3], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void()+theme(legend.text=element_text(size=10))

figure3b <- plot_grid(ggplot()+theme_void(),figure3b_9,
                      ggplot()+theme_void(),figure3b_8,
                      ggplot()+theme_void(),figure3b_7, 
                      figure3b_6,ggplot()+theme_void(),
                      figure3b_5,ggplot()+theme_void(), 
                      ggplot()+theme_void(),figure3b_4,
                      figure3b_3,ggplot()+theme_void(), 
                      ggplot()+theme_void(),figure3b_2,
                      figure3b_1,ggplot()+theme_void(), 
                      align = "hv", nrow = 9,ncol=2 , axis = "rlbt")

figure3 <- plot_grid(figure3a,
                     figure3b,align = "h", nrow = 1,ncol=2 , axis = "rlbt",rel_widths = c(2.6, 1))


pdf(paste(FIGURESFOLDER,"/FIGURE3.pdf",sep=""),height=8,width=14)
print(figure3)
dev.off() 

################## FIGURE 4 ##############################################################
modelTsurfLatAdj<-summary(lm(LatAdj~TSurf,data=DATA))
a<- modelTsurfLatAdj$coefficients[1]
b<- modelTsurfLatAdj$coefficients[2]
c<- modelTsurfLatAdj$coefficients[3]*sqrt(length(DATA[,1]))
d<- modelTsurfLatAdj$coefficients[4]*sqrt(length(DATA[,1]))

fm<-paste0("y = ",round(a,1),round(b,1),"x \n R^2 = ",
           round(modelTsurfLatAdj$r.squared,2),
           "\n p-value: < 2.2e-16")

figure4<- ggplot() +
  geom_point(data = DATA,aes(y = LatAdj, x = TSurf ,color=DATA$lakeTypeName),size=0.1)+
  geom_line(aes(x=c(0:30), y= a+b*c(0:30)))+
  geom_smooth(method= "lm")+
  geom_text(x=20,aes(y=60,label=fm),size=3)+
  scale_color_manual(name = "Legend",
                     values=clmap[c(1,2,3,4,8,5,7,6,9)],
                     breaks=lakeType[c(1,2,3,4,5,6,7,8,9)])+
  ylab("Adjusted Latitude (?N)")+
  xlab("Surface temperature (?C)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "white",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right")

pdf(paste(FIGURESFOLDER,"/FIGURE4.pdf",sep=""),height=6,width=6)
print(figure4)
dev.off() 

################## FIGURE 5 ##############################################################  
dftest<-oneway_test(formula=depthHL~ lakeTypeName, data = DATA, alternative="two.sided")
PM_A = pairwisePermutationMatrix(depthHL ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_A <- multcompLetters(PM_A$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_A),file=paste(ANALYSISFOLDER,"/stratA_depth.csv",sep=""))
write.csv(LABELS_A$Letters,file=paste(ANALYSISFOLDER,"/stratA_depth_labels.csv",sep=""))
fig5a<-      ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = depthHL , fill=lakeTypeName))+
  scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                labels = trans_format("log10", math_format(10^.x))) +
  geom_text(aes(x=c(1:9), y=max(DATA_dum$depthHL)*1.5,label=LABELS_A$Letters[c(6,7,8,9,1,4,2,5,3)]))+
  ylab("Depth (m)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "bottom",
        axis.title.x = element_blank(),
        axis.text.x = element_blank(),
        legend.text=element_text(size=14))+
  scale_fill_manual(name = "",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("a. Depth (log)"))


dftest<-oneway_test(formula=area ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_B = pairwisePermutationMatrix(area ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_B <- multcompLetters(PM_B$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_B),file=paste(ANALYSISFOLDER,"/stratB_area.csv",sep=""))
write.csv(LABELS_B$Letters,file=paste(ANALYSISFOLDER,"/stratB_area_labels.csv",sep=""))
fig5b<-     ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = area , fill=lakeTypeName))+
  scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                labels = trans_format("log10", math_format(10^.x))) +
  geom_text(aes(x=c(1:9), y=max(DATA_dum$area)*1.5,label=LABELS_B$Letters[c(6,7,8,9,1,4,2,5,3)]))+
  ylab(expression("Area (m"^2*")")) +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("b. Area (log)"))


dftest<-oneway_test(formula=shore_dev ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_C = pairwisePermutationMatrix(shore_dev ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_C <- multcompLetters(PM_C$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_C),file=paste(ANALYSISFOLDER,"/stratC_shore_dev.csv",sep=""))
write.csv(LABELS_C$Letters,file=paste(ANALYSISFOLDER,"/stratC_shore_dev_labels.csv",sep=""))
fig5c<-     ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = shore_dev , fill=lakeTypeName))+
  scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                labels = trans_format("log10", math_format(10^.x))) +
  geom_text(aes(x=c(1:9), y=max(DATA_dum$shore_dev)*1.5,label=LABELS_C$Letters[c(6,7,8,9,1,4,2,5,3)]))+
  ylab(expression("Shoreline development (-)")) +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("c. Shoreline development (log)"))


dftest<-oneway_test(formula=TEpi ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_D = pairwisePermutationMatrix(TEpi ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_D <- multcompLetters(PM_D$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_D),file=paste(ANALYSISFOLDER,"/stratD_TEpi.csv",sep=""))
write.csv(LABELS_D$Letters,file=paste(ANALYSISFOLDER,"/stratD_TEpi_labels.csv",sep=""))
fig5d<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = TEpi , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=max(DATA_dum$TEpi)*1.1,label=LABELS_D$Letters[c(6,7,8,9,1,4,2,5,3)]))+
  ylab("Temperature epilimnion (?C)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("d. Temperature epilimnion"))


dftest<-oneway_test(formula=THyp ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_E = pairwisePermutationMatrix(THyp ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_E <- multcompLetters(PM_E$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_E),file=paste(ANALYSISFOLDER,"/stratE_THyp.csv",sep=""))
write.csv(LABELS_E$Letters,file=paste(ANALYSISFOLDER,"/stratE_THyp_labels.csv",sep=""))
fig5e<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = THyp , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=max(DATA_dum$THyp)*1.1,label=LABELS_E$Letters[c(6,7,8,9,1,4,2,5,3)]))+
  ylab("Temperature hypolimnion (?C)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("e. Temperature hypolimnion"))


dftest<-oneway_test(formula=IceDays ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_F = pairwisePermutationMatrix(IceDays ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_F <- multcompLetters(PM_F$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_F),file=paste(ANALYSISFOLDER,"/stratF_IceDays.csv",sep=""))
write.csv(LABELS_F$Letters,file=paste(ANALYSISFOLDER,"/stratF_IceDays_labels.csv",sep=""))
fig5f<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat, y = IceDays , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=max(DATA_dum$IceDays)*1.02,label=LABELS_F$Letters[c(6,7,8,9,1,4,2,5,3)]))+   
  ylab("Number of ice days (days)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("f. Ice days"))


dftest<-oneway_test(formula=clay_T ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_G = pairwisePermutationMatrix(clay_T ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_G <- multcompLetters(PM_G$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_G),file=paste(ANALYSISFOLDER,"/stratG_clay_T.csv",sep=""))
write.csv(LABELS_G$Letters,file=paste(ANALYSISFOLDER,"/stratG_clay_T_labels.csv",sep=""))
fig5g<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat,  y = clay_T , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=max(DATA_dum$clay_T)*1.03,label=LABELS_G$Letters[c(6,7,8,9,1,4,2,5,3)]))+   
  ylab("Clay in sediments (%)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("g. Clay in sediments"))


dftest<-oneway_test(formula=sand_T ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_H = pairwisePermutationMatrix(sand_T ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_H <- multcompLetters(PM_H$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_H),file=paste(ANALYSISFOLDER,"/stratI_sand_T .csv",sep=""))   
write.csv(LABELS_H$Letters,file=paste(ANALYSISFOLDER,"/stratI_sand_T_labels.csv",sep=""))
fig5h<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat,  y = sand_T , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=100,label=LABELS_H$Letters[c(6,7,8,9,1,4,2,5,3)]))+   
  ylab("Sand in sediments (%)") +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "bottom",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ggtitle(paste("h. Sand in sediments"))


dftest<-oneway_test(formula=bulk_ref_T ~ lakeTypeName, data = DATA, alternative="two.sided")
PM_I = pairwisePermutationMatrix(bulk_ref_T ~ lakeTypeName,
                                 data = DATA,
                                 method="fdr")
LABELS_I <- multcompLetters(PM_I$Adjusted,
                            compare="<",
                            threshold=0.05,
                            Letters=LETTERS,
                            reversed = FALSE)
write.csv(as.data.frame(PM_I),file=paste(ANALYSISFOLDER,"/stratH_bulk_ref_T.csv",sep=""))  
write.csv(LABELS_I$Letters,file=paste(ANALYSISFOLDER,"/stratI_bulk_ref_T_labels.csv",sep=""))
fig5i<-   ggplot() + 
  geom_boxplot(data = DATA_dum,aes(x=newStrat,  y = bulk_ref_T , fill=lakeTypeName))+
  geom_text(aes(x=c(1:9), y=max(DATA_dum$bulk_ref_T)*1.02,label=LABELS_I$Letters[c(6,7,8,9,1,4,2,5,3)]))+   
  ylab(expression("Ref. Bulk Density (kg/dm"^3*")")) +
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "right",
        axis.title.x = element_blank(),
        axis.text.x = element_blank())+
  scale_fill_manual(name = "Legend",
                    values=clmap[c(1,2,3,4,8,5,7,6,9)],
                    breaks=lakeType[c(1:9)])+
  ylim(c(1.2,1.83))+
  ggtitle(paste("i. Reference bulk density"))




fig5_legend<-g_legend(fig5a)


pdf(paste(FIGURESFOLDER,"/FIGURE5.pdf",sep=""),height=13,width=13)
grid.arrange(arrangeGrob(fig5a + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5b + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5c + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5d + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5e + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5f + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5g + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5h + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         fig5i + theme(legend.position="none",text = element_text(size=13),axis.text= element_text(size=14)),
                         nrow=3,ncol=3),
             fig5_legend, nrow=2,heights=c(10, 1))
dev.off() 

################## STATISTICS MAIN TEXT ##############################################################  

#Figure 1 stats
#number of lakes in the east and west
sum(EASTlakes>0)/19538
sum(WESTlakes>0)/19538

# number of polymictic cont. warm lakes
sum(DATA$Stratification==2)/19538
# number of monomictic warm
sum(DATA$Stratification==6)/19538
# number of polymictic discont. warm lakes
sum(DATA$Stratification==4)/19538
# number of Amictic
sum(DATA$Stratification==8)/19538
# number poly cont cold lakes
sum(DATA$Stratification==1)/19538
# number poly discont cold lakes
sum(DATA$Stratification==3)/19538
# number monomictic cold
sum(DATA$Stratification==5)/19538
# dimictic
sum(DATA$Stratification==7)/19538
# meromictic
sum(DATA$Stratification==9)/19538

sum(which(DATA$LakeType==1)>0)/sum(which(DATA$area>0)>0)
sum(which(DATA$LakeType==2)>0)/sum(which(DATA$area>0)>0)
sum(which(DATA$LakeType==3)>0)/sum(which(DATA$area>0)>0)

length(which(DATA$depthHL<10))/length(DATA$VICid)
max(DATA$depthHL)

length(which(DATA$area<10))/length(DATA$VICid)
max(DATA$area)

length(which(DATA$shore_dev<2))/length(DATA$VICid)
max(DATA$shore_dev)

#Figure 2 stats
(sum(which(DATA_dum$newStrat==1)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==2)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==3)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==4)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==5)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==6)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==7)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==8)>0))/length(DATA$VICid)*100
(sum(which(DATA_dum$newStrat==9)>0))/length(DATA$VICid)*100

#lakes with depth less than 10m
sum(DATA$depthHL<10)/ sum(DATA$depthHL>0)

################ Appendix ############################################################
Appendix_S_strat_common_incl<- data.frame(DATA$`GFDL-ESM2M_strat_method2`,
                                          DATA$`HadGEM2-ES_strat_method2`,
                                          DATA$`IPSL-CM5A-LR_strat_method2`,
                                          DATA$MIROC5_strat_method2)
Appendix_S_strat_common_perc_incl <- data.frame(DATA$`GFDL-ESM2M_strat_perc_method2` ,
                                                DATA$`HadGEM2-ES_strat_perc_method2`,
                                                DATA$`IPSL-CM5A-LR_strat_perc_method2`,
                                                DATA$MIROC5_strat_perc_method2)
clmap_appendix <- c("ivory4","yellow2","red","orange","olivedrab","lightpink","orchid","royalblue","turquoise")


#Figure D1############################################################################
LK<-1:length(Appendix_S_strat_common_perc_incl[,1])
Appendix_data_perc<-data.frame(rbind(
  cbind(LK,sort(Appendix_S_strat_common_perc_incl[,1],decreasing = TRUE),1),
  cbind(LK,sort(Appendix_S_strat_common_perc_incl[,2],decreasing = TRUE),2),
  cbind(LK,sort(Appendix_S_strat_common_perc_incl[,3],decreasing = TRUE),3),
  cbind(LK,sort(Appendix_S_strat_common_perc_incl[,4],decreasing = TRUE),4)))
colnames(Appendix_data_perc)<-c('A_lake','A_data','A_model')

modelAgreement<-(length(LK)-sum(which(rowSums(Appendix_S_strat_common_incl)/4==Appendix_S_strat_common_incl[,1])>1))/length(LK)

figureD1<- ggplot(data=Appendix_data_perc,aes(
  x=A_lake/max(A_lake)*100,
  y=A_data*100,group=MODELS[A_model]))+
  geom_line(data=Appendix_data_perc,aes(color=MODELS[A_model]),size=1)+
  xlab('Percentage of lakes (%)')+
  ylab('Years within main stratification regime (%)')+
  theme(panel.background = element_rect(fill = "white",
                                        colour = "black",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "gray"),
        legend.title = element_blank(),
        legend.position = "right")+
  xlim(c(0,100))+
  ylim(c(0,100))

pdf(paste(FIGURESFOLDER,"/APPENDIX_FIGURED1.pdf",sep=""),height=4,width=6)
print(figureD1)
dev.off() 



#Figure D2############################################################################
appendix_strat<-1:length(Appendix_S_strat_common_incl[,1])
uniqx<-c(1:9)
for (i in 1:length(appendix_strat)){
  appendix_strat[i]<-uniqx[which.max(tabulate(match(Appendix_S_strat_common_incl[i,], uniqx)))]
}



newStrat<-appendix_strat
newStrat[which(appendix_strat==1)]<- 1
newStrat[which(appendix_strat==2)]<- 2
newStrat[which(appendix_strat==3)]<- 3
newStrat[which(appendix_strat==4)]<- 4
newStrat[which(appendix_strat==5)]<- 6
newStrat[which(appendix_strat==6)]<- 8
newStrat[which(appendix_strat==7)]<- 7
newStrat[which(appendix_strat==8)]<- 5
newStrat[which(appendix_strat==9)]<- 9

DATA_dum<-data.frame(lakeType[newStrat],newStrat)
colnames(DATA_dum)<-c('lakeTypeName','newStrat')

figD2lab <- array(0,c(7,1))
figD2lab[1]<- sum(which(DATA_dum$newStrat==1)>0)
figD2lab[2]<- sum(which(DATA_dum$newStrat==2)>0)
figD2lab[3]<- sum(which(DATA_dum$newStrat==3)>0)
figD2lab[4]<- sum(which(DATA_dum$newStrat==4)>0)
figD2lab[5]<- sum(which(DATA_dum$newStrat==5)>0)
figD2lab[6]<- sum(which(DATA_dum$newStrat==6)>0)
figD2lab[7]<- sum(which(DATA_dum$newStrat==7)>0)
figD2lab[8]<- sum(which(DATA_dum$newStrat==8)>0)
figD2lab[9]<- sum(which(DATA_dum$newStrat==9)>0)


#figure D2a
figureD2a<-ggplot() + 
  geom_bar(data = DATA_dum,aes(x=newStrat ,fill=DATA_dum$lakeTypeName)) +
  scale_y_log10(breaks = trans_breaks("log10", function(x) 10^x),
                labels = trans_format("log10", math_format(10^.x)))   +
  scale_x_continuous(breaks =c(1,2,3,4,5,6,7,8,9) ,
                     labels = lakeType)+
  theme(panel.background = element_rect(fill = "white",
                                        colour = "white",
                                        size = 0, linetype = "solid"),
        panel.grid.major = element_line(size = 0, 
                                        linetype = 'solid',
                                        colour = "white"),
        legend.position = "none",
        axis.text.x = element_text(angle=0, size=16,hjust=1),
        axis.text.y = element_text(size=16),
        axis.title =element_text(size=16))+
  scale_fill_manual(name = "Legend",
                    values=clmap_appendix[c(1,2,3,4,8,5,7,6,9)],
                    breaks=DATA_dum$lakeType[c(1,2,3,4,5,6,7,8,9)])+
  geom_text(aes(x=c(1,2,3,4,5,6,7,8,9),y=c(figD2lab*2)),label=figD2lab,size=5)+
  ylab("Number of lakes (-)")+ 
  xlab("")+
  coord_flip()



#Figure D2b
data_fraction_1 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==1)/sum(DATA_dum$newStrat==1),
                                           sum(DATA_dum[WESTlakes,]$newStrat==1)/sum(DATA_dum$newStrat==1))))
data_fraction_1$ymax = cumsum(data_fraction_1$fract)
data_fraction_1$ymin = c(0, head(data_fraction_1$ymax, n=-1))
data_fraction_1$label <- paste0(data_fraction_1$category, " (", round(data_fraction_1$fract*100,0),"%) ")
data_fraction_1$labelPosition <- (data_fraction_1$ymax + data_fraction_1$ymin) / 2
figure2b_D1 <- ggplot(data_fraction_1, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[5]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[5],lighten(clmap[5], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_2 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==2)/sum(DATA_dum$newStrat==2),
                                           sum(DATA_dum[WESTlakes,]$newStrat==2)/sum(DATA_dum$newStrat==2))))
data_fraction_2$ymax = cumsum(data_fraction_2$fract)
data_fraction_2$ymin = c(0, head(data_fraction_2$ymax, n=-1))
data_fraction_2$label <- paste0(data_fraction_2$category, " (", round(data_fraction_2$fract*100,0),"%) ")
data_fraction_2$labelPosition <- (data_fraction_2$ymax + data_fraction_2$ymin) / 2
figure2b_D2 <-ggplot(data_fraction_2, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[7]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[7],lighten(clmap[7], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_3 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==3)/sum(DATA_dum$newStrat==3),
                                           sum(DATA_dum[WESTlakes,]$newStrat==3)/sum(DATA_dum$newStrat==3))))
data_fraction_3$ymax = cumsum(data_fraction_3$fract)
data_fraction_3$ymin = c(0, head(data_fraction_3$ymax, n=-1))
data_fraction_3$label <- paste0(data_fraction_3$category, " (", round(data_fraction_3$fract*100,0),"%) ")
data_fraction_3$labelPosition <- (data_fraction_3$ymax + data_fraction_3$ymin) / 2
figure2b_D3 <- ggplot(data_fraction_3, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[6]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[6],lighten(clmap[6], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_4 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==4)/sum(DATA_dum$newStrat==4),
                                           sum(DATA_dum[WESTlakes,]$newStrat==4)/sum(DATA_dum$newStrat==4))))
data_fraction_4$ymax = cumsum(data_fraction_4$fract)
data_fraction_4$ymin = c(0, head(data_fraction_4$ymax, n=-1))
data_fraction_4$label <- paste0(data_fraction_4$category, " (", round(data_fraction_4$fract*100,0),"%) ")
data_fraction_4$labelPosition <- (data_fraction_4$ymax + data_fraction_4$ymin) / 2
figure2b_D4 <- ggplot(data_fraction_4, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[9]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[9],lighten(clmap[9], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

data_fraction_5 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==5)/sum(DATA_dum$newStrat==5),
                                           sum(DATA_dum[WESTlakes,]$newStrat==5)/sum(DATA_dum$newStrat==5))))
data_fraction_5$ymax = cumsum(data_fraction_5$fract)
data_fraction_5$ymin = c(0, head(data_fraction_5$ymax, n=-1))
data_fraction_5$label <- paste0(data_fraction_5$category, " (", round(data_fraction_5$fract*100,0),"%) ")
data_fraction_5$labelPosition <- (data_fraction_5$ymax + data_fraction_5$ymin) / 2
figure2b_D5 <- ggplot(data_fraction_5, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[1]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[1],lighten(clmap[1], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))

# please note that there were no lakes in this cathegory so that the scripts for figure 2b_d6 has been reduced
figure2b_D6 <- ggplot() +
  geom_rect(colour = clmap[4]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[4],lighten(clmap[4], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_7 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==7)/sum(DATA_dum$newStrat==7),
                                           sum(DATA_dum[WESTlakes,]$newStrat==7)/sum(DATA_dum$newStrat==7))))
data_fraction_7$ymax = cumsum(data_fraction_7$fract)
data_fraction_7$ymin = c(0, head(data_fraction_7$ymax, n=-1))
data_fraction_7$label <- paste0(data_fraction_7$category, " (", round(data_fraction_7$fract*100,0),"%) ")
data_fraction_7$labelPosition <- (data_fraction_7$ymax + data_fraction_7$ymin) / 2
figure2b_D7 <- ggplot(data_fraction_7, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[2]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[2],lighten(clmap[2], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_8 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==8)/sum(DATA_dum$newStrat==8),
                                           sum(DATA_dum[WESTlakes,]$newStrat==8)/sum(DATA_dum$newStrat==8))))
data_fraction_8$ymax = cumsum(data_fraction_8$fract)
data_fraction_8$ymin = c(0, head(data_fraction_8$ymax, n=-1))
data_fraction_8$label <- paste0(data_fraction_8$category, " (", round(data_fraction_8$fract*100,0),"%) ")
data_fraction_8$labelPosition <- (data_fraction_8$ymax + data_fraction_8$ymin) / 2
figure2b_D8 <- ggplot(data_fraction_8, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[8]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[8],lighten(clmap[8], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void() +theme(legend.text=element_text(size=10))


data_fraction_9 <- data.frame(list(category=c("East","West"),
                                   fract=c(sum(DATA_dum[EASTlakes,]$newStrat==9)/sum(DATA_dum$newStrat==9),
                                           sum(DATA_dum[WESTlakes,]$newStrat==9)/sum(DATA_dum$newStrat==9))))
data_fraction_9$ymax = cumsum(data_fraction_9$fract)
data_fraction_9$ymin = c(0, head(data_fraction_9$ymax, n=-1))
data_fraction_9$label <- paste0(data_fraction_9$category, " (", round(data_fraction_9$fract*100,0),"%) ")
data_fraction_9$labelPosition <- (data_fraction_9$ymax + data_fraction_9$ymin) / 2
figure2b_D9 <- ggplot(data_fraction_9, aes(ymax=ymax, ymin=ymin, xmax=4.5, xmin=1.5, fill=label)) +
  geom_rect(colour = clmap[3]) +
  coord_polar(theta = "y") + xlim(c(0, 4.5)) +
  scale_fill_manual(name = "",values=c(clmap[3],lighten(clmap[3], amount=0.7)))+
  coord_polar(theta="y") + 
  theme_void()+theme(legend.text=element_text(size=10))

figureD2b <- plot_grid(ggplot()+theme_void(),figure2b_D9,
                       ggplot()+theme_void(),figure2b_D8,
                       ggplot()+theme_void(),figure2b_D7, 
                       figure2b_D6,ggplot()+theme_void(),
                       figure2b_D5,ggplot()+theme_void(), 
                       ggplot()+theme_void(),figure2b_D4,
                       figure2b_D3,ggplot()+theme_void(), 
                       ggplot()+theme_void(),figure2b_D2,
                       figure2b_D1,ggplot()+theme_void(), 
                       align = "hv", nrow = 9,ncol=2 , axis = "rlbt")

figureD2 <- plot_grid(figureD2a,
                      figureD2b,align = "h", nrow = 1,ncol=2 , axis = "rlbt",rel_widths = c(2.6, 1))

pdf(paste(FIGURESFOLDER,"/APPENDIX_FIGURES2.pdf",sep=""),height=8,width=14)
print(figureD2)
dev.off() 

################## PCLAKE TABLE  ##############################################################  

DATAtablePCLAKE_T_Epi<- readRDS(paste(ANALYSISFOLDER,"/DATAtablePCLAKE_T_Epi.csv",sep=""))
DATAtablePCLAKE_T_Hyp<- readRDS(paste(ANALYSISFOLDER,"/DATAtablePCLAKE_T_Hyp.csv",sep=""))
DATAtableStratClass<- readRDS(paste(ANALYSISFOLDER,"/DATAtableStratClass.csv",sep=""))

PCLake<-data.frame(matrix(0,9,16))
mEpi<-data.frame(matrix(0,9,365))
mHyp<-data.frame(matrix(0,9,365))
mSurf<-data.frame(matrix(0,9,365))
for (SF in 1:9 ){
  print(array(DATA$lakeTypeName[which(DATA$Stratification==SF)][1]))
  PCLake[SF,1]<- array(DATA$lakeTypeName[which(DATA$Stratification==SF)][1])
  
  PCLake[SF,2]<-round(median(DATA$TSurf[which(DATA$Stratification==SF)]),2)  
  PCLake[SF,3]<-(round(median(DATA$TSurfMax[which(DATA$Stratification==SF)]),2)-round(median(DATA$TSurfMin[which(DATA$Stratification==SF)]),2))/2
  
  PCLake[SF,4]<-round(median(DATA$TEpi[which(DATA$Stratification==SF)]),2)  
  PCLake[SF,5]<-(round(median(DATA$TEpiMax[which(DATA$Stratification==SF)]),2)-round(median(DATA$TEpiMin[which(DATA$Stratification==SF)]),2))/2
  
  PCLake[SF,6]<-round(median(DATA$THyp[which(DATA$Stratification==SF)]),2)  
  PCLake[SF,7]<-(round(median(DATA$THypMax[which(DATA$Stratification==SF)]),2)-round(median(DATA$THypMin[which(DATA$Stratification==SF)]),2))/2
  
  
  PCLake[SF,8]<-round(median(DATA$LKdepth[which(DATA$Stratification==SF)]),2)
  
  PCLake[SF,9]<-round(sqrt(median(DATA$area[which(DATA$Stratification==SF)]))*1000,2)
  PCLake[SF,10]<-round(sqrt(median(DATA$area[which(DATA$Stratification==SF)]))*median(DATA$shore_dev[which(DATA$Stratification==SF)])*1000,2)
  
  PCLake[SF,11]<-round((median(DATA$Dis_avg[which(DATA$Stratification==SF)])/(median(DATA$area[which(DATA$Stratification==SF)])*1000000))*60*60*24*1000,2) #mm/d
  
  PCLake[SF,12]<-round(median(DATA$clay_T[which(DATA$Stratification==SF)]),2)
  
  PCLake[SF,13]<-round(median(DATA$Albedo[which(DATA$Stratification==SF)]),2)
  
  PCLake[SF,14]<-round(median(DATA$IceDays[which(DATA$Stratification==SF)]),0)
  
  PCLake[SF,15]<- round(median(DATA$StratDepth[which(DATA$Stratification==SF)]),2)
  
  Str_mean <-c(unlist(DATAtableStratClass[which(DATAtableStratClass1$Stratification==SF),3:366]))
  str_dum<-unlist(Str_mean[which(Str_mean>0)])
  PCLake[SF,16]<- median(str_dum)
  
  Surf_mean <-DATAtablePCLAKE_T_Surf[which(DATAtablePCLAKE_T_Surf$Stratification==SF),3:367]
  mSurf[SF,]<- apply(Surf_mean, 2, mean)
  
  Tepi_mean <-DATAtablePCLAKE_T_Epi[which(DATAtablePCLAKE_T_Epi$Stratification==SF),3:367]
  mEpi[SF,]<- apply(Tepi_mean, 2, mean)
  
  Thyp_mean <-DATAtablePCLAKE_T_Hyp[which(DATAtablePCLAKE_T_Hyp$Stratification==SF),3:367]
  mHyp[SF,]<- apply(Thyp_mean, 2, mean)
  
  
}
colnames(PCLake)<-c("Stratification type",'Tsurf','TsurfVar',"TEpi","TEpiVar","THyp","THypVar",
                    "depth","fetch","fetch_adj","dis",'lutum','albedo','Ice',"StratDepth","modStratPeriod")

mSurf_PL<-do.call(rbind, replicate(100, t(mSurf), simplify=FALSE))
mEpi_PL<-do.call(rbind, replicate(100, t(mEpi), simplify=FALSE))
mHyp_PL<-do.call(rbind, replicate(100, t(mHyp), simplify=FALSE))

colnames(mSurf_PL) <- c(PCLake[,1])
colnames(mEpi_PL) <- c(PCLake[,1])
colnames(mHyp_PL) <- c(PCLake[,1])

write.csv(mSurf_PL,paste(ANALYSISFOLDER,"/mSurf_PL.csv",sep=""))
write.csv(mEpi_PL,paste(ANALYSISFOLDER,"/mEpi_PL.csv",sep=""))
write.csv(mHyp_PL,paste(ANALYSISFOLDER,"/mHyp_PL.csv",sep=""))
write.csv(PCLake,paste(ANALYSISFOLDER,"/PCLake.csv",sep=""))

################## STATISTICS APPENDIX ##############################################################  

# percentage of lakes that always stay in one cathegory
sum(which(Appendix_data_perc$A_data==1)>0)/length(Appendix_data_perc[,1])*100

# percentage of lakes that always stay in one cathegory
sum(which(Appendix_data_perc$A_data>0.75)>0)/length(Appendix_data_perc[,1])*100

# percentage of lakes that always stay in one cathegory
sum(which(Appendix_data_perc$A_data>0.5)>0)/length(Appendix_data_perc[,1])*100

# percentage of agreement between models
sum(which(abs(rowMeans(Appendix_S_strat_common_incl)-floor(rowMeans(Appendix_S_strat_common_incl)))>0)>0)/length(Appendix_S_strat_common_incl[,1])*100


